/**************************************************************************/
/* File: ul_usb1.c - UL_USB1 board specific part of USB<->CAN converter   */
/*                                                                        */
/* LinCAN - (Not only) Linux CAN bus driver                               */
/* Copyright (C) 2002-2011 DCE FEE CTU Prague <http://dce.felk.cvut.cz>   */
/* Copyright (C) 2008 Jan Kriz email:johen@post.cz                        */
/*                                                                        */
/* LinCAN is free software; you can redistribute it and/or modify it      */
/* under terms of the GNU General Public License as published by the      */
/* Free Software Foundation; either version 2, or (at your option) any    */
/* later version.  LinCAN is distributed in the hope that it will be      */
/* useful, but WITHOUT ANY WARRANTY; without even the implied warranty    */
/* of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU    */
/* General Public License for more details. You should have received a    */
/* copy of the GNU General Public License along with LinCAN; see file     */
/* COPYING. If not, write to the Free Software Foundation, 675 Mass Ave,  */
/* Cambridge, MA 02139, USA.                                              */
/*                                                                        */
/* To allow use of LinCAN in the compact embedded systems firmware        */
/* and RT-executives (RTEMS for example), main authors agree with next    */
/* special exception:                                                     */
/*                                                                        */
/* Including LinCAN header files in a file, instantiating LinCAN generics */
/* or templates, or linking other files with LinCAN objects to produce    */
/* an application image/executable, does not by itself cause the          */
/* resulting application image/executable to be covered by                */
/* the GNU General Public License.                                        */
/* This exception does not however invalidate any other reasons           */
/* why the executable file might be covered by the GNU Public License.    */
/* Publication of enhanced or derived LinCAN files is required although.  */
/**************************************************************************/

#include "./can/can.h"
#include "./can/can_sysdep.h"
#include "./can/main.h"
#include "./can/devcommon.h"
#include "./can/setup.h"
// #include "./can/finish.h"
#include "./can/i82527.h"
//#include "../include/sja1000.h"
#include "./can/sja1000p.h"

#include "./can/errno.h"

#include "./can/ul_usb1.h"

/* Get a minor range for your devices from the usb maintainer */
#define USB_SKEL_MINOR_BASE	192

#define CAN_OP_MASK 0x80
#define CAN_OP_READ 0x80
#define CAN_OP_WRITE 0x00


	/* our private defines. if this grows any larger, use your own .h file */
#define MAX_TRANSFER		(PAGE_SIZE - 512)
/* MAX_TRANSFER is chosen so that the VM is not stressed by
   allocations > PAGE_SIZE and the number of packets in a page
   is an integer 512 is the largest possible packet on EHCI */
#define WRITES_IN_FLIGHT	8
/* arbitrarily chosen */

/* Define these values to match your devices */
#define USB_SKEL_VENDOR_ID	0xDEAD
#define USB_SKEL_PRODUCT_ID	0x1001


/*
 * IO_RANGE is the io-memory range that gets reserved, please adjust according
 * your hardware. Example: #define IO_RANGE 0x100 for i82527 chips or
 * #define IO_RANGE 0x20 for sja1000 chips in basic CAN mode.
 */
#define IO_RANGE 0x100

/** ul_usb1_request_io
 * ul_usb1_request_io: - reserve io or memory range for can board
 * @candev: pointer to candevice/board which asks for io. Field @io_addr
 *	of @candev is used in most cases to define start of the range
 *
 * The function ul_usb1_request_io() is used to reserve the io-memory. If your
 * hardware uses a dedicated memory range as hardware control registers you
 * will have to add the code to reserve this memory as well.
 * %IO_RANGE is the io-memory range that gets reserved, please adjust according
 * your hardware. Example: #define IO_RANGE 0x100 for i82527 chips or
 * #define IO_RANGE 0x20 for sja1000 chips in basic CAN mode.
 * Return Value: The function returns zero on success or %-ENODEV on failure
 * File: src/ul_usb1.c
 */
int ul_usb1_request_io(struct candevice_t *candev)
{
	return 0;
}

/** ul_usb1_release_io
 * ul_usb1_release_io - free reserved io memory range
 * @candev: pointer to candevice/board which releases io
 *
 * The function ul_usb1_release_io() is used to free reserved io-memory.
 * In case you have reserved more io memory, don't forget to free it here.
 * IO_RANGE is the io-memory range that gets released, please adjust according
 * your hardware. Example: #define IO_RANGE 0x100 for i82527 chips or
 * #define IO_RANGE 0x20 for sja1000 chips in basic CAN mode.
 * Return Value: The function always returns zero
 * File: src/ul_usb1.c
 */
int ul_usb1_release_io(struct candevice_t *candev)
{
	return 0;
}

/** ul_usb1_reset
 * ul_usb1_reset - hardware reset routine
 * @candev: Pointer to candevice/board structure
 *
 * The function ul_usb1_reset() is used to give a hardware reset. This is
 * rather hardware specific so I haven't included example code. Don't forget to
 * check the reset status of the chip before returning.
 * Return Value: The function returns zero on success or %-ENODEV on failure
 * File: src/ul_usb1.c
 */
int ul_usb1_reset(struct candevice_t *candev)
{
	return can_init();
}

#define RESET_ADDR 0x0
#define NR_82527 0
#define NR_SJA1000 1

/** ul_usb1_init_hw_data
 * ul_usb1_init_hw_data - Initialize hardware cards
 * @candev: Pointer to candevice/board structure
 *
 * The function ul_usb1_init_hw_data() is used to initialize the hardware
 * structure containing information about the installed CAN-board.
 * %RESET_ADDR represents the io-address of the hardware reset register.
 * %NR_82527 represents the number of Intel 82527 chips on the board.
 * %NR_SJA1000 represents the number of Philips sja1000 chips on the board.
 * The flags entry can currently only be %CANDEV_PROGRAMMABLE_IRQ to indicate that
 * the hardware uses programmable interrupts.
 * Return Value: The function always returns zero
 * File: src/ul_usb1.c
 */
int ul_usb1_init_hw_data(struct candevice_t *candev)
{
	candev->res_addr=RESET_ADDR;
	candev->nr_82527_chips=NR_82527;
	candev->nr_sja1000_chips=NR_SJA1000;
	candev->nr_all_chips=NR_82527+NR_SJA1000;
	candev->flags |= CANDEV_PROGRAMMABLE_IRQ*0;

	return 0;
}

/** ul_usb1_init_chip_data
 * ul_usb1_init_chip_data - Initialize chips
 * @candev: Pointer to candevice/board structure
 * @chipnr: Number of the CAN chip on the hardware card
 *
 * The function ul_usb1_init_chip_data() is used to initialize the hardware
 * structure containing information about the CAN chips.
 * %CHIP_TYPE represents the type of CAN chip. %CHIP_TYPE can be "i82527" or
 * "sja1000".
 * The @chip_base_addr entry represents the start of the 'official' memory map
 * of the installed chip. It's likely that this is the same as the @io_addr
 * argument supplied at module loading time.
 * The @clock entry holds the chip clock value in Hz.
 * The entry @sja_cdr_reg holds hardware specific options for the Clock Divider
 * register. Options defined in the %sja1000.h file:
 * %sjaCDR_CLKOUT_MASK, %sjaCDR_CLK_OFF, %sjaCDR_RXINPEN, %sjaCDR_CBP, %sjaCDR_PELICAN
 * The entry @sja_ocr_reg holds hardware specific options for the Output Control
 * register. Options defined in the %sja1000.h file:
 * %sjaOCR_MODE_BIPHASE, %sjaOCR_MODE_TEST, %sjaOCR_MODE_NORMAL, %sjaOCR_MODE_CLOCK,
 * %sjaOCR_TX0_LH, %sjaOCR_TX1_ZZ.
 * The entry @int_clk_reg holds hardware specific options for the Clock Out
 * register. Options defined in the %i82527.h file:
 * %iCLK_CD0, %iCLK_CD1, %iCLK_CD2, %iCLK_CD3, %iCLK_SL0, %iCLK_SL1.
 * The entry @int_bus_reg holds hardware specific options for the Bus
 * Configuration register. Options defined in the %i82527.h file:
 * %iBUS_DR0, %iBUS_DR1, %iBUS_DT1, %iBUS_POL, %iBUS_CBY.
 * The entry @int_cpu_reg holds hardware specific options for the cpu interface
 * register. Options defined in the %i82527.h file:
 * %iCPU_CEN, %iCPU_MUX, %iCPU_SLP, %iCPU_PWD, %iCPU_DMC, %iCPU_DSC, %iCPU_RST.
 * Return Value: The function always returns zero
 * File: src/ul_usb1.c
 */
int ul_usb1_init_chip_data(struct candevice_t *candev, int chipnr)
{
	/*i82527_fill_chipspecops(candev->chip[chipnr]);*/
	/*sja1000_fill_chipspecops(candev->chip[chipnr]);*/
	sja1000p_fill_chipspecops(candev->chip[chipnr]);

	candev->chip[chipnr]->flags|= CHIP_IRQ_CUSTOM;

	candev->chip[chipnr]->chip_base_addr=0;
	candev->chip[chipnr]->clock = 24000000;
	candev->chip[chipnr]->int_cpu_reg = iCPU_DSC;
	candev->chip[chipnr]->int_clk_reg = iCLK_SL1;
	candev->chip[chipnr]->int_bus_reg = iBUS_CBY;
	candev->chip[chipnr]->sja_cdr_reg = sjaCDR_CBP | sjaCDR_CLK_OFF;
	candev->chip[chipnr]->sja_ocr_reg = sjaOCR_MODE_NORMAL |
								sjaOCR_TX0_LH;

	candev->chip[chipnr]->chip_data=(void *)malloc(sizeof(struct ul_usb1_chip_data));
	if (candev->chip[chipnr]->chip_data==NULL)
		return -ENOMEM;
	return 0;
}

/** ul_usb1_init_obj_data
 * ul_usb1_init_obj_data - Initialize message buffers
 * @chip: Pointer to chip specific structure
 * @objnr: Number of the message buffer
 *
 * The function ul_usb1_init_obj_data() is used to initialize the hardware
 * structure containing information about the different message objects on the
 * CAN chip. In case of the sja1000 there's only one message object but on the
 * i82527 chip there are 15.
 * The code below is for a i82527 chip and initializes the object base addresses
 * The entry @obj_base_addr represents the first memory address of the message
 * object. In case of the sja1000 @obj_base_addr is taken the same as the chips
 * base address.
 * Unless the hardware uses a segmented memory map, flags can be set zero.
 * Return Value: The function always returns zero
 * File: src/ul_usb1.c
 */
int ul_usb1_init_obj_data(struct canchip_t *chip, int objnr)
{
	chip->msgobj[objnr]->obj_base_addr=chip->chip_base_addr+(objnr+1)*0x10;

	return 0;
}

/** ul_usb1_program_irq
 * ul_usb1_program_irq - program interrupts
 * @candev: Pointer to candevice/board structure
 *
 * The function ul_usb1_program_irq() is used for hardware that uses
 * programmable interrupts. If your hardware doesn't use programmable interrupts
 * you should not set the @candevices_t->flags entry to %CANDEV_PROGRAMMABLE_IRQ and
 * leave this function unedited. Again this function is hardware specific so
 * there's no example code.
 * Return value: The function returns zero on success or %-ENODEV on failure
 * File: src/ul_usb1.c
 */
int ul_usb1_program_irq(struct candevice_t *candev)
{
	return 0;
}

/** ul_usb1_write_register
 * ul_usb1_write_register - Low level write register routine
 * @data: data to be written
 * @address: memory address to write to
 *
 * The function ul_usb1_write_register() is used to write to hardware registers
 * on the CAN chip. You should only have to edit this function if your hardware
 * uses some specific write process.
 * Return Value: The function does not return a value
 * File: src/ul_usb1.c
 */
void ul_usb1_write_register(unsigned data, unsigned long address)
{
	can_write(data, address & 0xFF);
}

/** ul_usb1_read_register
 * ul_usb1_read_register - Low level read register routine
 * @address: memory address to read from
 *
 * The function ul_usb1_read_register() is used to read from hardware registers
 * on the CAN chip. You should only have to edit this function if your hardware
 * uses some specific read process.
 * Return Value: The function returns the value stored in @address
 * File: src/ul_usb1.c
 */
unsigned ul_usb1_read_register(unsigned long address)
{
	uint8_t data;
	data = can_read(address & 0xFF);
	return data;
}

/* !!! Don't change this function !!! */
int ul_usb1_register(struct hwspecops_t *hwspecops)
{
	hwspecops->request_io = ul_usb1_request_io;
	hwspecops->release_io = ul_usb1_release_io;
	hwspecops->reset = ul_usb1_reset;
	hwspecops->init_hw_data = ul_usb1_init_hw_data;
	hwspecops->init_chip_data = ul_usb1_init_chip_data;
	hwspecops->init_obj_data = ul_usb1_init_obj_data;
	hwspecops->write_register = ul_usb1_write_register;
	hwspecops->read_register = ul_usb1_read_register;
	hwspecops->program_irq = ul_usb1_program_irq;
	return 0;
}

